@@ -36,6 +36,18 @@
 #'                                split = " ")
 #' x <- document_term_frequencies(x = brussels_reviews$feedback, document = brussels_reviews$id, 
 #'                                split = "[[:space:][:punct:][:digit:]]+")
+#'                                
+#' ##
+#' ## document-term-frequencies on several fields to easily include bigram and trigrams
+#' ##
+#' library(data.table)
+#' x <- as.data.table(brussels_reviews_anno)
+#' x <- x[, token_bigram := txt_nextgram(token, n = 2), by = list(doc_id, sentence_id)]
+#' x <- x[, token_trigram := txt_nextgram(token, n = 3), by = list(doc_id, sentence_id)]
+#' x <- document_term_frequencies(x = x, 
+#'                                document = "doc_id", 
+#'                                term = c("token", "token_bigram", "token_trigram"))
+#' head(x)
 document_term_frequencies <- function(x, document, ...){
   UseMethod("document_term_frequencies")
 }
@@ -43,9 +55,19 @@ document_term_frequencies <- function(x, document, ...){
 #' @describeIn document_term_frequencies Create a data.frame with one row per document/term combination indicating the frequency of the term in the document
 #' @export
 document_term_frequencies.data.frame <- function(x, document = colnames(x)[1], term = colnames(x)[2], ...){
+  result <- list()
+  for(field in term){
+    result[[field]] <- dtf(x, document, term = field, ...)
+  }
+  result <- data.table::rbindlist(result)
+  result <- dtf(result)
+  result
+}
+
+dtf <- function(x, document = colnames(x)[1], term = colnames(x)[2], ...){
   # r cmd check happiness
   freq <- .N <- NULL
-
+  
   x <- setDT(x)
   if("freq" %in% colnames(x)){
     result <- x[, list(freq = as.integer(sum(freq))), by = c(document, term)]  
@@ -62,6 +84,7 @@ document_term_frequencies.data.frame <- function(x, document = colnames(x)[1], t
   result
 }
 
+
 #' @describeIn document_term_frequencies Create a data.frame with one row per document/term combination indicating the frequency of the term in the document
 #' @param split The regular expression to be used if \code{x} is a character vector. 
 #' This will split the character vector \code{x} in pieces by the provides split argument. 
@@ -108,6 +131,19 @@ document_term_frequencies.character <- function(x, document=paste("doc", seq_alo
 #' ## allowing you to making sure terms which are not in the data are provided in the resulting dtm
 #' allterms <- unique(x$term)
 #' dtm <- document_term_matrix(head(x, 1000), vocabulary = allterms)
+#' 
+#' ##
+#' ## Example adding bigrams/trigrams to the document term matrix
+#' ## Mark that this can also be done using ?dtm_cbind
+#' ##
+#' library(data.table)
+#' x <- as.data.table(brussels_reviews_anno)
+#' x <- x[, token_bigram := txt_nextgram(token, n = 2), by = list(doc_id, sentence_id)]
+#' x <- x[, token_trigram := txt_nextgram(token, n = 3), by = list(doc_id, sentence_id)]
+#' x <- document_term_frequencies(x = x, 
+#'                                document = "doc_id", 
+#'                                term = c("token", "token_bigram", "token_trigram"))
+#' dtm <- document_term_matrix(x)
 document_term_matrix <- function(x, vocabulary, ...){
   UseMethod("document_term_matrix")
 }
@@ -169,7 +205,6 @@ document_term_matrix.simple_triplet_matrix <- function(x, ...){
 
 
 
-
 #' @title Inverse operation of the document_term_matrix function
 #' @description Inverse operation of the \code{\link{document_term_matrix}} function. 
 #' Creates frequency table which contains 1 row per document/term
@@ -354,4 +389,91 @@ dtm_cor <- function(x) {
   covmat <- (as.matrix(Matrix::crossprod(x)) - n * Matrix::tcrossprod(Matrix::colMeans(x))) / (n - 1)
   cormat <- covmat / Matrix::tcrossprod(sqrt(Matrix::diag(covmat)))
   cormat
+}
+
+
+#' @title Combine 2 document term matrices either by rows or by columns
+#' @description These 2 methods provide \code{\link{cbind}} and \code{\link{rbind}} functionality 
+#' for sparse matrix objects which are returned by \code{\link{document_term_matrix}}. \cr
+#' 
+#' In case of \code{dtm_cbind}, if the rows are not ordered in the same way in x and y, it will order them based on the rownames. 
+#' If there are missing rows these will be filled with NA values. \cr
+#' In case of \code{dtm_rbind}, if the columns are not ordered in the same way in x and y, it will order them based on the colnames. 
+#' If there are missing columns these will be filled with NA values.
+#' @param x a sparse matrix such as a "dgTMatrix" object which is returned by \code{\link{document_term_matrix}}
+#' @param y a sparse matrix such as a "dgTMatrix" object which is returned by \code{\link{document_term_matrix}}
+#' @return a sparse matrix where either rows are put below each other in case of \code{dtm_rbind}
+#' or columns are put next to each other in case of \code{dtm_cbind}
+#' @seealso \code{\link{document_term_matrix}}
+#' @name dtm_bind
+#' @aliases dtm_rbind dtm_cbind
+#' @export
+#' @examples 
+#' data(brussels_reviews_anno)
+#' x <- brussels_reviews_anno
+#' 
+#' ## rbind
+#' dtm1 <- document_term_frequencies(x = subset(x, doc_id %in% c("10049756", "10284782")))
+#' dtm1 <- document_term_matrix(dtm1, document = "doc_id", term = c("token"))
+#' dtm2 <- document_term_frequencies(x = subset(x, doc_id %in% c("10789408", "12285061", "35509091")))
+#' dtm2 <- document_term_matrix(dtm2, document = "doc_id", term = c("token"))
+#' m <- dtm_rbind(dtm1, dtm2)
+#' dim(m)
+#' 
+#' ## cbind
+#' library(data.table)
+#' x <- as.data.table(brussels_reviews_anno)
+#' x <- x[, token_bigram := txt_nextgram(token, n = 2), by = list(doc_id, sentence_id)]
+#' dtm1 <- document_term_frequencies(x = x, document = "doc_id", term = c("token"))
+#' dtm1 <- document_term_matrix(dtm1)
+#' dtm2 <- document_term_frequencies(x = x, document = "doc_id", term = c("token_bigram"))
+#' dtm2 <- document_term_matrix(dtm2)
+#' m <- dtm_cbind(dtm1, dtm2)
+#' dim(m)
+#' m <- dtm_cbind(dtm1[-c(100, 999), ], dtm2[-1000,])
+#' dim(m)
+dtm_cbind <- function(x, y){
+  if(is.null(rownames(x))) stop("x needs to have rownames")
+  if(is.null(rownames(y))) stop("y needs to have rownames")
+  if(length(intersect(colnames(x), colnames(y))) > 0) stop("x and y should not have overlapping column names")
+  rowsleft <- rownames(x)
+  rowsright <- rownames(y)
+  r <- union(rowsleft, rowsright)
+  addleft <- setdiff(rowsright, rowsleft)
+  if(length(addleft) > 0){
+    addleft <- Matrix::sparseMatrix(i = integer(), j = integer(), x = NA,
+                                    dims = c(length(addleft), ncol(x)), dimnames = list(addleft, colnames(x)))
+    x <- methods::rbind2(x, addleft)
+  }
+  addright <- setdiff(rowsleft, rowsright)
+  if(length(addright) > 0){
+    addright <- Matrix::sparseMatrix(i = integer(), j = integer(), x = NA,
+                                     dims = c(length(addright), ncol(y)), dimnames = list(addright, colnames(y)))
+    y <- methods::rbind2(y, addright)
+  }
+  cbind2(x[r, ], y[r, ])
+}
+
+#' @export
+#' @rdname dtm_bind
+dtm_rbind <- function(x, y){
+  if(is.null(colnames(x))) stop("x needs to have colnames")
+  if(is.null(colnames(y))) stop("y needs to have colnames")
+  if(length(intersect(rownames(x), rownames(y))) > 0) stop("x and y should not have overlapping row names")
+  colsleft <- colnames(x)
+  colsright <- colnames(y)
+  r <- union(colsleft, colsright)
+  addleft <- setdiff(colsright, colsleft)
+  if(length(addleft) > 0){
+    addleft <- Matrix::sparseMatrix(i = integer(), j = integer(), x = NA,
+                                    dims = c(nrow(x), length(addleft)), dimnames = list(rownames(x), addleft))
+    x <- methods::cbind2(x, addleft)
+  }
+  addright <- setdiff(colsleft, colsright)
+  if(length(addright) > 0){
+    addright <- Matrix::sparseMatrix(i = integer(), j = integer(), x = NA,
+                                     dims = c(nrow(y), length(addright)), dimnames = list(rownames(y), addright))
+    y <- methods::cbind2(y, addright)
+  }
+  rbind2(x[, r], y[, r])
 }
\ No newline at end of file
